package pl.edu.agh.kis.eis.ndl

case class Placeholder[A](s: VarOrAtom[A]*) {
  def isBound: Boolean = s.forall({
    case v: Variable[A] => v.binding.isDefined
    case _: Const[A] => true
  })

  def buildFact(): MonoFact[A] = MonoFact(s.map {
    case v: Variable[A] => v.binding.get
    case Const(a) => a
  }: _*)

  def unify(fact: MonoFact[A]): Seq[A] = {
    val placeholderVars = s.zipWithIndex.collect { case (v: Variable[A], index: Int) => (v, index) }.toSet
    placeholderVars.foreach { case (variable, index) => variable.bind(fact.values(index)) }

    if (buildFact() == fact) {
      placeholderVars.toSeq.map(_._1.binding.get)
    } else {
      Seq.empty
    }
  }

  def isBindingCompatible(binding: Set[Variable[A]]): Boolean = {
    variables.filter(v => v.isBound && binding.exists(_.name == v.name)).forall(binding.contains)
  }

  lazy val arity: Int = s.size // FIXME common with MonoFact
  lazy val constants: Seq[(Int, A)] = s.zipWithIndex.collect { case (Const(const), index) => (index, const) }
  lazy val variables: Set[Variable[A]] = s.collect { case v: Variable[A] => v }.toSet
}

