package pl.edu.agh.kis.eis.ndl

sealed trait VarOrAtom[A]

case class Variable[A](var binding: Option[A] = None, name: Option[String] = None) extends VarOrAtom[A] {
  def isBound: Boolean = binding.isDefined

  def bind(value: A): A = {
    binding = Some(value)
    value
  }
}

object Variable {
  def apply[A](value: A): Variable[A] = Variable(Some(value))

  def apply[A](name: String): Variable[A] = Variable(name = Some(name))

  def apply[A](name: String, value: A): Variable[A] = Variable(Some(value), Some(name))

  def augmentBindings[A](knownBindings: Seq[Set[Variable[A]]], candidate: Set[Variable[A]]): Seq[Set[Variable[A]]] = {
    if (knownBindings.isEmpty) {
      return Seq(candidate)
    }

    val compatible = knownBindings.filter { knownBinding =>
      val shared = knownBinding.filter(x => candidate.exists(_.name == x.name))
      shared.forall(candidate.contains)
    }
    compatible.map(_ ++ candidate)
  }
}

case class Const[A](a: A) extends VarOrAtom[A]